.\" $Id: //depot/main/rmanprod/docs/renderman_pro_server/prman_technical_rendering/man/shadeop.3#1 $ (Pixar) $Date: 2004/06/16 $
.TH SHADEOP 3S "$Date: 2004/06/16 $" "Release 3.0" "\fIPhotoRealistic RenderMan\fP (UNIX)"
.SH NAME
ShadeBeginFrame, ShadeBindGOP, ShadeCleanupThread, ShadeEnd, ShadeEndFrame,
ShadeGrid, ShadeInit, ShadeInitThread, ShadePrint, ShadeServerEndFrame,
ShadeTexturePrint, ShadeTextureStatsInit, ShadeTreeInit, ShaderPrint
.SH SYNOPSIS
.nf
.ft B
#include <reyes.h>
#include <renderctx.h>
.PP
.ft B
ShadeBeginFrame(int maxgridsize)
.PP
.ft B
ShadeBindGOP(struct gops *gop, struct vertex_vector *vv)
.PP
.ft B
ShadeCleanupThread(ShadeContext *ctx)
.PP
.ft B
ShadeEnd()
.PP
.ft B
ShadeEndFrame()
.PP
.ft B
ShadeGrid(ShadeContext *ctx, struct grid *g)
.PP
.ft B
ShadeInit()
.PP
.ft B
ShadeInitThread(ShadeContext *ctx, int id, int maxgridsize)
.PP
.ft B
ShadePrint(struct stinstance *sti)
.PP
.ft B
ShadeServerEndFrame()
.PP
.ft B
ShadeTexturePrint()
.PP
.ft B
ShadeTextureStatsInit()
.PP
.ft B
RtPointer
ShadeTreeInit(char *name, ShadeType riptype, int nparams, char *paramname[],
		float *paramvalue[], Mxstruct *mxp)
.PP
.ft B
ShaderPrint(char *s)
.SH DESCRIPTION
.PP
.I ShadeInit
must be called at the "beginning of time".
It is responsible for calling various shading subsystem initialization
procedures (\fIShadeTextureInit()\fP, \fIShadeServerInit()\fP)
and for licensed versions \fIrlm_poll()\fP to establish license
validity.
.PP
.I ShadeBeginFrame
must be called at the beginning of a frame
(when maxgridsize has been determined).
It allocates (relatively) static data structures (global conditional
data structure, zero register, null register).
It sets the
.I ShadeInFrame
flag (for freeing shaders at end of frame),
the number of color components and frame starting time.
It calls \fIShadeServerBeginFrame()\fP, \fIShadeTextureBeginFrame()\fP, and
\fIShadeInstanceCacheBeginFrame()\fP
to perform subsystem specific frame initialization.
.PP
.I ShadeInitThread
allocates and binds context-specific data for a shader thread.
It must be called at least once for each thread of execution
(and after any calls to \fIShadeCleanupThread()\fP)
to prepare a new thread for execution.
.I ctx
is a pointer to the thread "context",
.I id
is a small integer identifying the context, and
.I maxgridsize
is the maximum number of points in any grid this thread is expected
to handle.
(The current scheme allocates contexts on the stack.
This procedure should be responsible for allocating the context
and returning a pointer to it.)
.PP
.I ShadeTreeInit
binds the specified parameters and transformation matrix ("shader")
to the specified shader.
It is called for every RI \fIshader directive\fP.
It calls
.I ShadeFindShader()
to locate a copy of the shader definition and
.I sic_MakeEntry()
to make/find the entry in the "instance cache".
.PP
.I ShadeBindGOP
binds surface, geometry, and atmosphere shader instances to a
particular geometric primitive.
This involves calling
.I ShadeBindShadertoGOP()
for each shader
to determine if there are varying parameters, and if so,
calling
.I sic_FindVertRef()
to make/find an entry in the "bound instance" cache.
If there are varying parameters, \fIShadeBindGOP\fP will
update the appropriate pointers in the \fIGOP\fP
to point to the "bound instance" entry.
Therefore, if the \fIGOP\fP is to be copied, the copy should
be made after calling \fIShadeBindGOP\fP.
.PP
.I ShadeGrid
performs any required "late" binding, allocates any memory
required for shader variables and evaluates the geometry shader,
light shaders, surface shader, and atmosphere shader for
the primitive associated with the grid.
.PP
.I ShadeCleanupThread
frees resources allocated and bound by
.I ShadeInitThread.
(It should also free the context which should be allocated by
.I ShadeInitThread
)
.PP
.I ShadeEndFrame
calls
.I shadecleanupstats()
to free statistics resources,
clears the
.I ShadeInFrame
flag, calls
.I ShadeTextureEndFrame()
,
.I ShadeInstanceCacheEndFrame()
and frees resources allocated
by
.I ShadeBeginFrame().
.PP
.I ShadeServerEndFrame
is called at end of frame when it is safe to free shaders allocated
during the frame.
It should probably be called automatically from
.I ShadeEndFrame()
(as
.I ShadeServerBeginFrame()
is called from
.I ShadeBeginFrame()
), but due to multiprocessor rendezvous complications, this
must be called explicitly.
.PP
.I ShadePrint
prints a textual representation of a shader from a shader instance.
.PP
.I ShadeTexturePrint
calls
.I TextureStatsPrint()
to print texture sub-system statistics.
.PP
.I ShadeTextureStatsInit
should allocate any resources required to gather texture statistics.
(Currently a no-op.)
.PP
.I ShaderPrint
calls \fIShadeStatsPrint()\fP if \fIs\fP is the string "stat"
to print shading subsystem statistics.
.PP
.I ShadeEnd
should be called at the end of time.
(Currently a no-op.)
.PP
The interface lacks (at least) the following functions:
.TS
l lw(60).
ShadeIncInstanceRef	T{
increment the reference count on a shader instance.
T}
ShadeDecInstanceRef	T{
decrement the reference count on a shader instance.
T}
.TE
.SH EXTERNS
The shading subsystem makes the following external references:
.TS
l lw(12) lw(45).
Refernce	From	Description
_
Duff12To8Linear[]	txinput.c	T{
table to convert pixel values from 12 to 8 bits.
T}
EyeToScreenNDC()	cstrans.c	T{
obsolete calls for \fItoscreen()\fP and \fItoeye()\fP.
T}
EyeToWorld()	cstrans.c	\^
EyeToWorldTransform	txr_shad.c	T{
returns pointer to the \fIEyeToWorld\fP transform.
T}
FileOpen	shadin.c	T{
does a \fIfopen()\fP given a name, path, and extension.
T}
GenerateIVector	shadexec.c	T{
generates the \fII\fP vector given \fIP\fP and the number of points.
T}
GridFindUserVar	shadexec.c	T{
given a hashed (or tokenized) variable name and a pointer
to a grid, returns a pointer to the associated \fIvertex variable\fP data.
T}
HitherYon	depth.c	T{
returns values of near and far clipping planes (by reference)
for \fIdepth\fP shadeop.
T}
MxInvert	T{
cstrans.c, shadexec.c
T}	T{
given pointers to Mxstructs, returns the inverse of
the first in the second.
T}
MxMultiply	T{
txr_shad.c, cstrans.c
T}	T{
given pointers to three Mxstructs, returns product
of the first two in the third.
T}
MxReference	T{
cstrans.c, sesic.c
T}	T{
Identity Mxstruct.
T}
ObjectToEye	_point.c	T{
transform some number of points from object to eye space
T}
PicClose	txinput.c	T{
close a picio file.
T}
PicOpen	txinput.c	T{
open a picio file.
T}
RIPNewToken	cstrans.c	T{
tokenize a coordinate system name for
\fItransform()\fP.
T}
RI_CAMERA	cstrans.c	T{
tokenized coordinate system name for \fItransform()\fP.
T}
RI_CS	shadexec.c	T{
tokenized name for vertex variable lookup.
T}
RI_N	shadexec.c	T{
tokenized name for vertex variable lookup.
T}
RI_OBJECT	cstrans.c	T{
tokenized coordinate system name for \fItransform()\fP.
T}
RI_OS	shadexec.c	T{
tokenized name for vertex variable lookup.
T}
RI_S	shadexec.c	T{
tokenized name for vertex variable lookup.
T}
RI_T	shadexec.c	T{
tokenized name for vertex variable lookup.
T}
RMSPortRead	txserver.c	T{
Read data from a communications port, \fITextureDispatch()\fP.
T}
RMSPortWrite	txserver.c	T{
Write data from a communications port, \fITextureDispatch()\fP.
T}
SurfacePath	shadin.c	T{
shader default search path, passed to \fIFileOpen()\fP from \fIShadeLoadShaderFile()\fP.
T}
TIFFClose	txinput.c	T{
close a TIFF file.
T}
TIFFGetField	txinput.c	T{
retrieve TIFF tag values.
T}
TIFFOpen	txinput.c	T{
open a TIFF file.
T}
TIFFReadScanline	txinput.c	T{
read data from a TIFF file.
T}
TIFFScanlineSize	txinput.c	T{
return size of a TIFF scanline.
T}
TexturePath	texture.c	T{
default texture search path, passed to \fITextureOpen()\fP from \fIShadeTextureOpen()\fP.
T}
Transform	T{
cstrans.c, ptform.c, shadexec.c
T}	T{
Transform some points given a transformation matrix.
T}
WorldToEye	_point.c	T{
transform some number of points from world to eye space.
T}
Xcpt	various	T{
Standard error handler.
T}
_RI_PolygonUV	shadexec.c	T{
tokenized name for vertex variable lookup.
T}
drand	txr_shad.c, randcol.c	T{
random number generator.
T}
drandinit	txr_shad.c	T{
initialize random number generator.
T}
get_from_to_matrix	cstrans.c	T{
given two tokenized coordinate system names,
build the transform from one to the other.
T}
picDecodeScanline	txinput.c	T{
read a picio file scanline.
T}
picPostDecodeScanline	txinput.c	T{
clean up picio file resources prior to close.
T}
picPreDecodeScanline	txinput.c	T{
setup picio file for access.
T}
ri_camera	shadexec.c	T{
retrieve camera shutter open time (for \fItime\fP graphics state variable).
T}
ri_getmarkedcoords	cstrans.c	T{
obsolete call for \fItonamed\fP.
T}
.TE
.SH FILES
.ta \w'%RMAN%/prman/lib/libshadeop.a    'u
.nf
%RMAN%/prman/lib/libshadeop.a	- shadeop library
.fi
.SH BUGS
The multi-threaded code could be better integrated into the
system.
